#[derive(Debug, PartialEq)]
enum ImageType {
    Jpeg,
    Png,
    Gif,
    WebP,
    AnimatedWebP,
    Unsupported,
}

// fn process_image(file_path_string: String) -> Result<String, String> {
//     // convert string to path to use in rust
//     let file_path = std::path::Path::new(&file_path_string);

//     // get the data directory path
//     let data_dir = tauri::api::path::data_dir().ok_or("Failed to get data directory")?;
//     let mut destination = data_dir.clone();
//     // we can hardcode the file name since
//     // 1. we are only dealing with one wallpaper image at a time
//     // 2. we will be converting all images to the webp format
//     destination.push("wallpaper.webp");

//     // determine the file format
// }

/// determines the image type of a file that is passed in using the filepath
fn determine_image_type(file_path: &std::path::Path) -> Result<ImageType, String> {
    // open the file
    let mut file =
        std::fs::File::open(file_path).map_err(|e| format!("Failed to open file: {e}"))?;

    // read the first few bytes to determine the format
    let mut buffer = [0; 12];
    std::io::Read::read_exact(&mut file, &mut buffer)
        .map_err(|e| format!("Failed to read file: {e}"))?;

    // with hopes and prayers, try to guess the format and pray that it is correct haha
    match image::guess_format(&buffer) {
        Ok(image::ImageFormat::Jpeg) => Ok(ImageType::Jpeg),
        Ok(image::ImageFormat::Png) => Ok(ImageType::Png),
        Ok(image::ImageFormat::Gif) => Ok(ImageType::Gif),
        Ok(image::ImageFormat::WebP) => {
            // additional check for webp
            is_animated_webp(file_path).map(|animated| {
                if animated {
                    ImageType::AnimatedWebP
                } else {
                    ImageType::WebP
                }
            })
        }
        _ => Ok(ImageType::Unsupported),
    }
}

/// check if the file is a valid webp image and if it is animated
/// returns `Ok(true)` if it is animated, `Ok(false)` if it is not
/// and errors out if the file cannot be read or is not a webp
fn is_animated_webp(file_path: &std::path::Path) -> Result<bool, String> {
    // open the file and read its contents into a buffer
    let mut buffer = Vec::new();
    std::fs::File::open(file_path)
        .and_then(|mut file| std::io::Read::read_to_end(&mut file, &mut buffer))
        .map_err(|e| format!("Failed to read file: {}", e))?;

    // use the webp crate to decode the image and check for animation
    webp::AnimDecoder::new(&buffer)
        .decode()
        .map(|anim| anim.has_animation())
        .map_err(|_| "File is not a valid WebP image".to_string())
}

/// crop static images (jpeg, png and still webp) to a maximum of 1280x800 and maintaining aspect ratio
fn crop_static_image(
    file_path: &std::path::Path,
    destination: &std::path::Path,
) -> Result<(), String> {
    let img = image::open(file_path).map_err(|e| format!("Failed to open image: {e}"))?;
    let (width, height) = image::GenericImageView::dimensions(&img);

    let resized_img = if width > 1280 || height > 800 {
        img.resize(1280, 800, image::imageops::FilterType::Lanczos3)
    } else {
        img
    };

    let mut output_file = std::fs::File::create(destination)
        .map_err(|e: std::io::Error| format!("Failed to create file: {e}"))?;
    resized_img
        .write_to(&mut output_file, image::ImageFormat::WebP)
        .map_err(|e| format!("Failed to save image as WebP: {}", e))?;
    Ok(())
}

/// crop animated webp images
fn crop_animated_webp(
    file_path: &std::path::Path,
    destination: &std::path::Path,
) -> Result<(), String> {
    let mut buffer = Vec::new();
    std::fs::File::open(file_path)
        .and_then(|mut file| std::io::Read::read_to_end(&mut file, &mut buffer))
        .map_err(|e| format!("Failed to read file: {}", e))?;

    let decoder = webp::AnimDecoder::new(&buffer);
    let decoded = decoder
        .decode()
        .map_err(|e| format!("Failed to decode WebP animation: {e}"))?;
}

fn main() {
    // Example usage
    let path = std::path::Path::new("/home/vomitblood/Downloads/title.keys");
    match determine_image_type(path) {
        Ok(image_type) => println!("Image type: {:?}", image_type),
        Err(e) => println!("Error determining image type: {}", e),
    }
}
